/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Mozilla Foundation.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Mark Banner <bugzilla@standard8.plus.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

MSGDBFOLDER_ATOM(kTotalUnreadMessagesAtom, "TotalUnreadMessages")
MSGDBFOLDER_ATOM(kBiffStateAtom, "BiffState")
MSGDBFOLDER_ATOM(kNewMailReceivedAtom, "NewMailReceived")
MSGDBFOLDER_ATOM(kNewMessagesAtom, "NewMessages")
MSGDBFOLDER_ATOM(kInVFEditSearchScopeAtom, "inVFEditSearchScope")
MSGDBFOLDER_ATOM(kNumNewBiffMessagesAtom, "NumNewBiffMessages")
MSGDBFOLDER_ATOM(kTotalMessagesAtom, "TotalMessages")
MSGDBFOLDER_ATOM(kFolderSizeAtom, "FolderSize")
MSGDBFOLDER_ATOM(kStatusAtom, "Status")
MSGDBFOLDER_ATOM(kFlaggedAtom, "Flagged")
MSGDBFOLDER_ATOM(kNameAtom, "Name")
MSGDBFOLDER_ATOM(kSynchronizeAtom, "Synchronize")
MSGDBFOLDER_ATOM(kOpenAtom, "open")
MSGDBFOLDER_ATOM(kIsDeferred, "isDeferred")
MSGDBFOLDER_ATOM(kKeywords, "Keywords")
MSGDBFOLDER_ATOM(mFolderLoadedAtom, "FolderLoaded")
MSGDBFOLDER_ATOM(mDeleteOrMoveMsgCompletedAtom, "DeleteOrMoveMsgCompleted")
MSGDBFOLDER_ATOM(mDeleteOrMoveMsgFailedAtom, "DeleteOrMoveMsgFailed")
MSGDBFOLDER_ATOM(mJunkStatusChangedAtom, "JunkStatusChanged")
MSGDBFOLDER_ATOM(mFiltersAppliedAtom, "FiltersApplied")
MSGDBFOLDER_ATOM(mFolderFlagAtom, "FolderFlag")
