/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim:set ts=2 sw=2 sts=2 et: */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Google Inc.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Darin Fisher <darin@meer.net>
 *  Dietrich Ayala <dietrich@mozilla.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// This tests the deprecated livemarks interface.

function run_test()
{
  let livemarkId = PlacesUtils.livemarks.createLivemarkFolderOnly(
    PlacesUtils.bookmarksMenuFolderId, "foo", uri("http://example.com/"),
    uri("http://example.com/rss.xml"), PlacesUtils.bookmarks.DEFAULT_INDEX
  );

  do_check_true(PlacesUtils.livemarks.isLivemark(livemarkId));
  do_check_eq(PlacesUtils.livemarks.getSiteURI(livemarkId).spec, "http://example.com/");
  do_check_eq(PlacesUtils.livemarks.getFeedURI(livemarkId).spec, "http://example.com/rss.xml");
  do_check_true(PlacesUtils.bookmarks.getFolderReadonly(livemarkId));

  // Make sure we can't add a livemark to a livemark
  let livemarkId2 = null;
  try {
    let livemarkId2 = PlacesUtils.livemarks.createLivemark(
      livemarkId, "foo", uri("http://example.com/"),
      uri("http://example.com/rss.xml"), PlacesUtils.bookmarks.DEFAULT_INDEX
    );
  } catch (ex) {
    livemarkId2 = null;
  }
  do_check_true(livemarkId2 == null);
  
  // make sure it didn't screw up the first one
  do_check_true(PlacesUtils.livemarks.isLivemark(livemarkId));

  do_check_eq(
    PlacesUtils.livemarks.getLivemarkIdForFeedURI(uri("http://example.com/rss.xml")),
    livemarkId
  );

  // make sure folders don't get counted as bookmarks
  // create folder
  let randomFolder = PlacesUtils.bookmarks.createFolder(
    PlacesUtils.bookmarksMenuFolderId, "Random",
    PlacesUtils.bookmarks.DEFAULT_INDEX
  );
  do_check_true(!PlacesUtils.livemarks.isLivemark(randomFolder));
}
